#!   /usr/bin/env   python
#    coding: utf8

# Copyright CERN, 2011
# Author: Matthieu Cattin <matthieu.cattin@cern.ch>
# Licence: GPL v2 or later.
# Website: http://www.ohwr.org

# Import system modules
import sys
import time
import os
import math
import i2c
import onewire
import ds18b20
import eeprom_24aa64

# Add common modules location tp path
sys.path.append('../../../')
sys.path.append('../../../gnurabbit/python/')
sys.path.append('../../../common/')

# Import common modules
import rr
from ptsexcept import *
import csr
import gn4124


"""
FMC TDC class
"""
class CFMCTDC:

    # Wishbone core adresses
    GNUM_CSR_ADDR = 0x0
    TDC_CORE_ADDR = 0x80000


    # Register addresses
    ACAM_READBACK_ADDR = [0x40, 0x44, 0x48, 0x4C, 0X50, 0x54, 0x58, 0x5C, 0x60, 0x64, 0x68, 0x6C, 0x70, 0x78]

    # DMA length in bytes
    DMA_LENGTH = 4096

    # Write pointer register
    WP_MASK = 0xFFF
    WP_OVERFLOW_MASK = 0xFFFFF000

    # Timestamp meta-data
    META_CHANNEL_MASK = 0xF
    META_SLOPE_MASK = 0xF0
    META_FIFO_LF_MASK = 0xF00
    META_FIFO_EF_MASK = 0xF000
    META_SLOPE = ['falling', 'rising']

    def __init__(self, bus):
        # Objects delcaration
        self.bus = bus;
        self.tdc_regs = csr.CCSR(self.bus, self.TDC_CORE_ADDR)
        self.gnum = gn4124.CGN4124(self.bus, self.GNUM_CSR_ADDR)

        # Set GNUM local bus frequency
        self.gnum.set_local_bus_freq(160)
        print "Gennum local bus clock %d MHz"%(self.gnum.get_local_bus_freq())
        time.sleep(2)

        # Reset FPGA
        print "Resetting FPGA"
        self.gnum.rstout33_cycle()
        time.sleep(3)

        # Get physical addresses of the memory pages for DMA transfer
        self.dma_pages = self.gnum.get_physical_addr()


###############################################################################################

    # Configures ACAM    
    def config_acam(self):
        print "\n--  --  --  --  --  -- configuring acam  --  --  --  --  --  --"
        print "Configuring ACAM"
        time.sleep(2)
        self.tdc_regs.wr_reg(0x00, 0x1F0FC81)
        self.tdc_regs.wr_reg(0x04, 0x0)
        self.tdc_regs.wr_reg(0x08, 0xE02)
        time.sleep(2)
        self.tdc_regs.wr_reg(0x0C, 0x0)
        self.tdc_regs.wr_reg(0x10, 0x200000F)
        self.tdc_regs.wr_reg(0x14, 0x7D0)
        self.tdc_regs.wr_reg(0x18, 0x3)
        self.tdc_regs.wr_reg(0x1C, 0x1FEA)
        self.tdc_regs.wr_reg(0x2C, 0xFF0000)
        self.tdc_regs.wr_reg(0x30, 0x4000000)
        self.tdc_regs.wr_reg(0x38, 0x0)
        self.tdc_regs.wr_reg(0xFC, 0x4)
        time.sleep(2)

    def load_acam_config(self):
        self.tdc_regs.wr_reg(0xFC, 0x4)
        time.sleep(2)

    def reset_acam(self):
        print "\n--  --  --  --  --  --  resetting acam   --  --  --  --  --  --"
        print "Reseting ACAM"
        self.tdc_regs.wr_reg(0xFC, 0x100)
        time.sleep(2)

    def readback_acam_config(self):
        print "\n--  --  --  --  reading back acam configuration  --  --  --  --"
        print "Reading back ACAM configuration."
        self.tdc_regs.wr_reg(0xFC, 0x8)
        time.sleep(2)
        for i in self.ACAM_READBACK_ADDR:
            print "reg 0x%02X = 0x%08X"%(i, self.tdc_regs.rd_reg(i))

    def read_acam_status(self):
        self.tdc_regs.wr_reg(0xFC, 0x10)
        return self.tdc_regs.rd_reg(0x70)

    def enable_channels(self):
        self.tdc_regs.wr_bit(0x84, 7, 1)

    def channel_term(self, channel, enable):
        self.tdc_regs.wr_bit(0x84, channel-1, enable)
        #print "0x%08X"%self.tdc_regs.rd_reg(0x84)

    def set_utc(self, utc):
        self.tdc_regs.wr_reg(0x80, utc)
        self.tdc_regs.wr_reg(0xFC, 0x200)

    def get_utc(self):
        return self.tdc_regs.rd_reg(0x94)

    def start_acq(self):
        print "\n--  --  --  --  --  --  --  --  --  --  --  --  --  --  --"
        print "Start acquisition"
        self.tdc_regs.wr_reg(0xFC, 0x1)

    def stop_acq(self):
        print "\n--  --  --  --  --  --  --  --  --  --  --  --  --  --  --"
        print "Stop acquisition"
        self.tdc_regs.wr_reg(0xFC, 0x2)

    def get_pointer(self):
        return (self.WP_MASK & self.tdc_regs.rd_reg(0x9C))

    def get_overflow_counter(self):
        return ((self.WP_OVERFLOW_MASK & self.tdc_regs.rd_reg(0x9C)) >> 12)

    def get_timestamps(self, verbose=0):
        # Read the number of bytes to be read from the board
        dma_length = self.get_pointer()
        carrier_addr = 0x0

        # Calculate the number of DMA item required to get the data
        items_required = int(math.ceil(dma_length/float(self.DMA_LENGTH)))
        if(128 < items_required):
            print('Required items: %d')%items_required
            raise Exception('Current gn4124 class only supports up to 128 items.')
        if(verbose):
            print('Required items: %d')%items_required

        # Configure DMA
        for num in range(items_required):
            if(items_required == num+1):
                next_item = 0
                item_length = (dma_length-(num*self.DMA_LENGTH))
            else:
                next_item = 1
                item_length = self.DMA_LENGTH
            if(0 == num):
                item_start_addr = carrier_addr
            else:
                item_start_addr = carrier_addr + (num*self.DMA_LENGTH)
            if(verbose):
                print("item nb:%d item_carrier_addr:0x%.8X item_host_addr:0x%.8X item_length:%d next_item:%d)")%(num,item_start_addr,self.dma_pages[num+1],item_length,next_item)
            self.gnum.add_dma_item(item_start_addr, self.dma_pages[num+1], item_length, 0, next_item)
        if(verbose):
            if(items_required > 1):
                items = self.gnum.get_memory_page(0)
                print('DMA items:')
                for i in range(items_required*8):
                    print('%.4X: %.8X')%(i*4,items[i])

        # Start DMA
        dma_finished = 0
        if(verbose):
            print "Start DMA"
        self.gnum.start_dma()

        """
        # Poll on DMA status
        print "Wait for DMA done"
        while('Done' == self.gnum.get_dma_status()):
            time.sleep(0.01)
        print "DMA done!"
        """

        # Wait for end of DMA interrupt
        if(verbose):
            print('Wait GN4124 interrupt')
        self.gnum.wait_irq()
        if(verbose):
            print('GN4124 interrupt occured')
        """
        print('irq mask:%.4X')%self.get_irq_en_mask() if(verbose)
        while(0 == dma_finished):
            irq_src = self.get_irq_source()
            if(verbose):
                print('IRQ source : %.4X')%irq_src
                print('DMA status: %s')%self.gnum.get_dma_status()
            if(irq_src & self.IRQ_SRC_DMA_END):
                print('IRQ source : %.4X')%irq_src if(verbose)
                self.clear_irq_source(self.IRQ_SRC_DMA_END)
                print('IRQ source : %.4X')%self.get_irq_source() if(verbose)
                dma_finished = 1
            time.sleep(0.005)
        print('DMA finished!') if(verbose)
        """
        # Retrieve data from host memory
        data = []
        for i in range(items_required):
            data += self.gnum.get_memory_page(i+1)
        if(verbose):
            print('data length:%d')%(len(data)*4)

        # Format data
        if(verbose):
            print "Data"
        raw_timestamps = []
        timestamps = []
        for i in range(0,dma_length/4,4):
            timestamp = data[i+2] * 1E12 + data[i+1] * 8E3 + data[i] * 81.03
            channel = self.META_CHANNEL_MASK & data[i+3]
            slope = ((self.META_SLOPE_MASK & data[i+3]) >> 4)
            fifo_lf = ((self.META_FIFO_LF_MASK & data[i+3]) >> 8)
            fifo_ef = ((self.META_FIFO_EF_MASK & data[i+3]) >> 12)
            raw_timestamps.append((data[i+3]<<96)+(data[i+2]<<64)+(data[i+1]<<32)+data[i])
            timestamps.append([timestamp, channel, slope, fifo_lf, fifo_ef])
            if(verbose):
                print "[%03d] sec:%15.3f channel:%d slope:%7s fifo_lf:%d fifo_ef:%d"%(i/4,timestamp, channel, self.META_SLOPE[slope], fifo_lf, fifo_ef)
                #print "[%03d] sec:%15.3f slope:%d fifo_lf:%d fifo_ef:%d"%(i/4,timestamp, slope, fifo_lf, fifo_ef)
        if(verbose):
            print "Number of timestamps = %d" % (len(timestamps))
            for i in range(len(timestamps)):
                print timestamps[i]

        return timestamps


if __name__ == '__main__' :
    main()
